@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -15,11 +14,13 @@
 import collections
 import itertools
 import json
+
+from oslo.utils import encodeutils
+from oslo.utils import strutils
 import six
-from heat.engine import constraints as constr
 
 from heat.common import exception
-from heat.openstack.common import strutils
+from heat.engine import constraints as constr
 
 
 PARAMETER_KEYS = (
@@ -48,13 +49,13 @@ class Schema(constr.Schema):
     # For Parameters the type name for Schema.LIST is CommaDelimitedList
     # and the type name for Schema.MAP is Json
     TYPES = (
-        STRING, NUMBER, LIST, MAP,
+        STRING, NUMBER, LIST, MAP, BOOLEAN,
     ) = (
-        'String', 'Number', 'CommaDelimitedList', 'Json',
+        'String', 'Number', 'CommaDelimitedList', 'Json', 'Boolean',
     )
 
     def __init__(self, data_type, description=None, default=None, schema=None,
-                 constraints=[], hidden=False, label=None):
+                 constraints=None, hidden=False, label=None):
         super(Schema, self).__init__(data_type=data_type,
                                      description=description,
                                      default=default,
@@ -65,7 +66,7 @@ class Schema(constr.Schema):
         self.hidden = hidden
 
     # Schema class validates default value for lists assuming list type. For
-    # comma delimited list string supported in paramaters Schema class, the
+    # comma delimited list string supported in parameters Schema class, the
     # default value has to be parsed into a list if necessary so that
     # validation works.
     def _validate_default(self, context):
@@ -75,16 +76,16 @@ class Schema(constr.Schema):
                 try:
                     default_value = self.default.split(',')
                 except (KeyError, AttributeError) as err:
-                    raise constr.InvalidSchemaError(_('Default must be a '
-                                                      'comma-delimited list '
-                                                      'string: %s') % str(err))
+                    raise exception.InvalidSchemaError(
+                        message=_('Default must be a comma-delimited list '
+                                  'string: %s') % err)
             try:
                 self.validate_constraints(default_value, context)
-            except (ValueError, TypeError) as exc:
-                raise constr.InvalidSchemaError(_('Invalid default '
-                                                  '%(default)s (%(exc)s)') %
-                                                dict(default=self.default,
-                                                     exc=exc))
+            except (ValueError, TypeError,
+                    exception.StackValidationFailed) as exc:
+                raise exception.InvalidSchemaError(
+                    message=_('Invalid default %(default)s (%(exc)s)') %
+                    dict(default=self.default, exc=exc))
 
     def set_default(self, default=None):
         super(Schema, self).set_default(default)
@@ -100,27 +101,35 @@ class Schema(constr.Schema):
     @staticmethod
     def _check_dict(schema_dict, allowed_keys, entity):
         if not isinstance(schema_dict, dict):
-            raise constr.InvalidSchemaError(
-                _("Invalid %s, expected a mapping") % entity)
+            raise exception.InvalidSchemaError(
+                message=_("Invalid %s, expected a mapping") % entity)
         for key in schema_dict:
             if key not in allowed_keys:
-                raise constr.InvalidSchemaError(
-                    _("Invalid key '%(key)s' for %(entity)s") % {
+                raise exception.InvalidSchemaError(
+                    message=_("Invalid key '%(key)s' for %(entity)s") % {
                         "key": key, "entity": entity})
 
     @classmethod
-    def _validate_dict(cls, schema_dict):
-        cls._check_dict(schema_dict, cls.PARAMETER_KEYS, "parameter")
+    def _validate_dict(cls, param_name, schema_dict):
+        cls._check_dict(schema_dict,
+                        cls.PARAMETER_KEYS,
+                        "parameter (%s)" % param_name)
 
         if cls.TYPE not in schema_dict:
-            raise constr.InvalidSchemaError(_("Missing parameter type"))
+            raise exception.InvalidSchemaError(
+                message=_("Missing parameter type for parameter: %s") %
+                param_name)
 
     @classmethod
-    def from_dict(cls, schema_dict):
+    def from_dict(cls, param_name, schema_dict):
         """
         Return a Parameter Schema object from a legacy schema dictionary.
+
+        :param param_name: name of the parameter owning the schema; used
+               for more verbose logging
+        :type  param_name: str
         """
-        cls._validate_dict(schema_dict)
+        cls._validate_dict(param_name, schema_dict)
 
         def constraints():
             desc = schema_dict.get(CONSTRAINT_DESCRIPTION)
@@ -148,7 +157,7 @@ class Schema(constr.Schema):
                                               'false')).lower() == 'true',
                    label=schema_dict.get(LABEL))
 
-    def validate_value(self, name, value, context=None):
+    def validate_value(self, value, context=None):
         super(Schema, self).validate_constraints(value, context)
 
     def __getitem__(self, key):
@@ -172,7 +181,7 @@ class Parameter(object):
 
         # Check for fully-fledged Schema objects
         if not isinstance(schema, Schema):
-            schema = Schema.from_dict(schema)
+            schema = Schema.from_dict(name, schema)
 
         if schema.type == schema.STRING:
             ParamClass = StringParam
@@ -182,6 +191,8 @@ class Parameter(object):
             ParamClass = CommaDelimitedListParam
         elif schema.type == schema.MAP:
             ParamClass = JsonParam
+        elif schema.type == schema.BOOLEAN:
+            ParamClass = BooleanParam
         else:
             raise ValueError(_('Invalid Parameter type "%s"') % schema.type)
 
@@ -206,13 +217,20 @@ class Parameter(object):
         '''
         self.schema.validate(context)
 
-        if validate_value:
+        if not validate_value:
+            return
+
+        try:
             if self.has_default():
                 self._validate(self.default(), context)
             if self.user_value is not None:
                 self._validate(self.user_value, context)
             elif not self.has_default():
                 raise exception.UserParameterMissing(key=self.name)
+        except exception.StackValidationFailed as ex:
+            msg = _("Parameter '%(name)s' is invalid: %(exp)s") % dict(
+                name=self.name, exp=six.text_type(ex))
+            raise exception.StackValidationFailed(message=msg)
 
     def value(self):
         '''Get the parameter value, optionally sanitising it for output.'''
@@ -222,7 +240,7 @@ class Parameter(object):
         if self.has_default():
             return self.default()
 
-        raise KeyError(_('Missing parameter %s') % self.name)
+        raise exception.UserParameterMissing(key=self.name)
 
     def has_value(self):
         '''Parameter has a user or default value.'''
@@ -257,7 +275,7 @@ class Parameter(object):
         if self.hidden():
             return '******'
         else:
-            return strutils.safe_encode(six.text_type(value))
+            return encodeutils.safe_encode(six.text_type(value))
 
 
 class NumberParam(Parameter):
@@ -272,20 +290,39 @@ class NumberParam(Parameter):
         return float(super(NumberParam, self).value())
 
     def _validate(self, val, context):
-        self.schema.validate_value(self.name, val, context)
+        try:
+            Schema.str_to_num(val)
+        except ValueError as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
+        self.schema.validate_value(val, context)
 
     def value(self):
+        return Schema.str_to_num(super(NumberParam, self).value())
+
+
+class BooleanParam(Parameter):
+    '''A template parameter of type "Boolean".'''
+
+    def _validate(self, val, context):
         try:
-            return int(self)
-        except ValueError:
-            return float(self)
+            strutils.bool_from_string(val, strict=True)
+        except ValueError as ex:
+            raise exception.StackValidationFailed(message=six.text_type(ex))
+        self.schema.validate_value(val, context)
+
+    def value(self):
+        if self.user_value is not None:
+            raw_value = self.user_value
+        else:
+            raw_value = self.default()
+        return strutils.bool_from_string(str(raw_value), strict=True)
 
 
 class StringParam(Parameter):
     '''A template parameter of type "String".'''
 
     def _validate(self, val, context):
-        self.schema.validate_value(self.name, val, context)
+        self.schema.validate_value(val, context)
 
 
 class CommaDelimitedListParam(Parameter, collections.Sequence):
@@ -299,13 +336,12 @@ class CommaDelimitedListParam(Parameter, collections.Sequence):
         # only parse when value is not already a list
         if isinstance(value, list):
             return value
-
         try:
-            if value:
+            if value is not None:
                 return value.split(',')
         except (KeyError, AttributeError) as err:
             message = _('Value must be a comma-delimited list string: %s')
-            raise ValueError(message % str(err))
+            raise ValueError(message % six.text_type(err))
         return value
 
     def value(self):
@@ -319,13 +355,18 @@ class CommaDelimitedListParam(Parameter, collections.Sequence):
         '''Return an item from the list.'''
         return self.parsed[index]
 
+    def __str__(self):
+        if self.hidden():
+            return super(CommaDelimitedListParam, self).__str__()
+        return ",".join(self.value())
+
     def _validate(self, val, context):
         parsed = self.parse(val)
-        self.schema.validate_value(self.name, parsed, context)
+        self.schema.validate_value(parsed, context)
 
 
-class JsonParam(Parameter, collections.Mapping):
-    """A template parameter who's value is valid map."""
+class JsonParam(Parameter):
+    """A template parameter who's value is map or list."""
 
     def __init__(self, name, schema, value=None):
         super(JsonParam, self).__init__(name, schema, value)
@@ -334,12 +375,12 @@ class JsonParam(Parameter, collections.Mapping):
     def parse(self, value):
         try:
             val = value
-            if isinstance(val, collections.Mapping):
+            if not isinstance(val, basestring):
                 val = json.dumps(val)
             if val:
                 return json.loads(val)
         except (ValueError, TypeError) as err:
-            message = _('Value must be valid JSON: %s') % str(err)
+            message = _('Value must be valid JSON: %s') % err
             raise ValueError(message)
         return value
 
@@ -355,9 +396,14 @@ class JsonParam(Parameter, collections.Mapping):
     def __len__(self):
         return len(self.parsed)
 
+    def __str__(self):
+        if self.hidden():
+            return super(JsonParam, self).__str__()
+        return encodeutils.safe_decode(json.dumps(self.value()))
+
     def _validate(self, val, context):
         val = self.parse(val)
-        self.schema.validate_value(self.name, val, context)
+        self.schema.validate_value(val, context)
 
 
 class Parameters(collections.Mapping):
@@ -372,11 +418,13 @@ class Parameters(collections.Mapping):
         'AWS::StackId', 'AWS::StackName', 'AWS::Region'
     )
 
-    def __init__(self, stack_identifier, tmpl, user_params={}):
+    def __init__(self, stack_identifier, tmpl, user_params=None):
         '''
         Create the parameter container for a stack from the stack name and
         template, optionally setting the user-supplied parameter values.
         '''
+        user_params = user_params or {}
+
         def user_parameter(schema_item):
             name, schema = schema_item
             return Parameter(name, schema,
@@ -386,7 +434,8 @@ class Parameters(collections.Mapping):
         self.user_params = user_params
 
         schemata = self.tmpl.param_schemata()
-        user_parameters = (user_parameter(si) for si in schemata.iteritems())
+        user_parameters = (user_parameter(si) for si in
+                           six.iteritems(schemata))
         pseudo_parameters = self._pseudo_parameters(stack_identifier)
 
         self.params = dict((p.name,
@@ -428,7 +477,7 @@ class Parameters(collections.Mapping):
         optional filter function) and return the resulting dictionary.
         '''
         return dict((n, func(p))
-                    for n, p in self.params.iteritems() if filter_func(p))
+                    for n, p in six.iteritems(self.params) if filter_func(p))
 
     def set_stack_id(self, stack_identifier):
         '''
@@ -453,8 +502,8 @@ class Parameters(collections.Mapping):
                 param = key
                 break
         if param is not None:
-            template_params = self.tmpl.t[key]
-            for name, attrs in template_params.iteritems():
+            template_params = self.tmpl.t[key] or {}
+            for name, attrs in six.iteritems(template_params):
                 if not isinstance(attrs, dict):
                     raise exception.InvalidTemplateParameter(key=name)
 
