@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -14,19 +13,24 @@
 
 import collections
 
+import six
+
 from heat.common import exception
+from heat.engine import constraints as constr
+from heat.engine import function
 from heat.engine import parameters
 from heat.engine import support
-from heat.engine import constraints as constr
 
 SCHEMA_KEYS = (
     REQUIRED, IMPLEMENTED, DEFAULT, TYPE, SCHEMA,
     ALLOWED_PATTERN, MIN_VALUE, MAX_VALUE, ALLOWED_VALUES,
     MIN_LENGTH, MAX_LENGTH, DESCRIPTION, UPDATE_ALLOWED,
+    IMMUTABLE,
 ) = (
     'Required', 'Implemented', 'Default', 'Type', 'Schema',
     'AllowedPattern', 'MinValue', 'MaxValue', 'AllowedValues',
     'MinLength', 'MaxLength', 'Description', 'UpdateAllowed',
+    'Immutable',
 )
 
 
@@ -41,22 +45,24 @@ class Schema(constr.Schema):
 
     KEYS = (
         TYPE, DESCRIPTION, DEFAULT, SCHEMA, REQUIRED, CONSTRAINTS,
-        UPDATE_ALLOWED
+        UPDATE_ALLOWED, IMMUTABLE,
     ) = (
         'type', 'description', 'default', 'schema', 'required', 'constraints',
-        'update_allowed'
+        'update_allowed', 'immutable',
     )
 
     def __init__(self, data_type, description=None,
                  default=None, schema=None,
-                 required=False, constraints=[],
+                 required=False, constraints=None,
                  implemented=True,
                  update_allowed=False,
+                 immutable=False,
                  support_status=support.SupportStatus()):
         super(Schema, self).__init__(data_type, description, default,
                                      schema, required, constraints)
         self.implemented = implemented
         self.update_allowed = update_allowed
+        self.immutable = immutable
         self.support_status = support_status
         # validate structural correctness of schema itself
         self.validate()
@@ -73,7 +79,8 @@ class Schema(constr.Schema):
 
         unknown = [k for k in schema_dict if k not in SCHEMA_KEYS]
         if unknown:
-            raise constr.InvalidSchemaError(_('Unknown key(s) %s') % unknown)
+            raise exception.InvalidSchemaError(
+                message=_('Unknown key(s) %s') % unknown)
 
         def constraints():
             def get_num(key):
@@ -94,7 +101,8 @@ class Schema(constr.Schema):
         try:
             data_type = schema_dict[TYPE]
         except KeyError:
-            raise constr.InvalidSchemaError(_('No %s specified') % TYPE)
+            raise exception.InvalidSchemaError(
+                message=_('No %s specified') % TYPE)
 
         if SCHEMA in schema_dict:
             if data_type == Schema.LIST:
@@ -103,11 +111,9 @@ class Schema(constr.Schema):
                 schema_dicts = schema_dict[SCHEMA].items()
                 ss = dict((n, cls.from_legacy(sd)) for n, sd in schema_dicts)
             else:
-                raise constr.InvalidSchemaError(_('%(schema)s supplied for '
-                                                  ' %(type)s %(data)s') %
-                                                dict(schema=SCHEMA,
-                                                     type=TYPE,
-                                                     data=data_type))
+                raise exception.InvalidSchemaError(
+                    message=_('%(schema)s supplied for %(type)s %(data)s') %
+                    dict(schema=SCHEMA, type=TYPE, data=data_type))
         else:
             ss = None
 
@@ -118,7 +124,8 @@ class Schema(constr.Schema):
                    required=schema_dict.get(REQUIRED, False),
                    constraints=list(constraints()),
                    implemented=schema_dict.get(IMPLEMENTED, True),
-                   update_allowed=schema_dict.get(UPDATE_ALLOWED, False))
+                   update_allowed=schema_dict.get(UPDATE_ALLOWED, False),
+                   immutable=schema_dict.get(IMMUTABLE, False))
 
     @classmethod
     def from_parameter(cls, param):
@@ -134,7 +141,8 @@ class Schema(constr.Schema):
             param.STRING: cls.STRING,
             param.NUMBER: cls.NUMBER,
             param.LIST: cls.LIST,
-            param.MAP: cls.MAP
+            param.MAP: cls.MAP,
+            param.BOOLEAN: cls.BOOLEAN
         }
 
         # make update_allowed true by default on TemplateResources
@@ -143,11 +151,32 @@ class Schema(constr.Schema):
                    description=param.description,
                    required=param.required,
                    constraints=param.constraints,
-                   update_allowed=True)
+                   update_allowed=True,
+                   immutable=False)
+
+    def allowed_param_prop_type(self):
+        """
+        Return allowed type of Property Schema converted from parameter.
+
+        Especially, when generating Schema from parameter, Integer Property
+        Schema will be supplied by Number parameter.
+        """
+        param_type_map = {
+            self.INTEGER: self.NUMBER,
+            self.STRING: self.STRING,
+            self.NUMBER: self.NUMBER,
+            self.BOOLEAN: self.BOOLEAN,
+            self.LIST: self.LIST,
+            self.MAP: self.MAP
+        }
+
+        return param_type_map[self.type]
 
     def __getitem__(self, key):
         if key == self.UPDATE_ALLOWED:
             return self.update_allowed
+        elif key == self.IMMUTABLE:
+            return self.immutable
         else:
             return super(Schema, self).__getitem__(key)
 
@@ -180,6 +209,9 @@ class Property(object):
     def update_allowed(self):
         return self.schema.update_allowed
 
+    def immutable(self):
+        return self.schema.immutable
+
     def has_default(self):
         return self.schema.default is not None
 
@@ -192,7 +224,7 @@ class Property(object):
     def support_status(self):
         return self.schema.support_status
 
-    def _validate_integer(self, value):
+    def _get_integer(self, value):
         if value is None:
             value = self.has_default() and self.default() or 0
         try:
@@ -202,19 +234,19 @@ class Property(object):
         else:
             return value
 
-    def _validate_number(self, value):
+    def _get_number(self, value):
         if value is None:
             value = self.has_default() and self.default() or 0
         return Schema.str_to_num(value)
 
-    def _validate_string(self, value):
+    def _get_string(self, value):
         if value is None:
             value = self.has_default() and self.default() or ''
         if not isinstance(value, basestring):
             raise ValueError(_('Value must be a string'))
         return value
 
-    def _validate_children(self, child_values, keys=None):
+    def _get_children(self, child_values, keys=None, validate=False):
         if self.schema.schema is not None:
             if keys is None:
                 keys = list(self.schema.schema)
@@ -222,30 +254,34 @@ class Property(object):
             properties = Properties(schemata, dict(child_values),
                                     parent_name=self.name,
                                     context=self.context)
-            properties.validate()
+            if validate:
+                properties.validate()
+
             return ((k, properties[k]) for k in keys)
         else:
             return child_values
 
-    def _validate_map(self, value):
+    def _get_map(self, value, validate=False):
         if value is None:
             value = self.has_default() and self.default() or {}
         if not isinstance(value, collections.Mapping):
             raise TypeError(_('"%s" is not a map') % value)
 
-        return dict(self._validate_children(value.iteritems()))
+        return dict(self._get_children(six.iteritems(value),
+                                       validate=validate))
 
-    def _validate_list(self, value):
+    def _get_list(self, value, validate=False):
         if value is None:
             value = self.has_default() and self.default() or []
         if (not isinstance(value, collections.Sequence) or
                 isinstance(value, basestring)):
             raise TypeError(_('"%s" is not a list') % repr(value))
 
-        return [v[1] for v in self._validate_children(enumerate(value),
-                                                      range(len(value)))]
+        return [v[1] for v in self._get_children(enumerate(value),
+                                                 range(len(value)),
+                                                 validate)]
 
-    def _validate_bool(self, value):
+    def _get_bool(self, value):
         if value is None:
             value = self.has_default() and self.default() or False
         if isinstance(value, bool):
@@ -256,25 +292,29 @@ class Property(object):
 
         return normalised == 'true'
 
-    def _validate_data_type(self, value):
+    def get_value(self, value, validate=False):
+        """Get value from raw value and sanitize according to data type."""
+
         t = self.type()
         if t == Schema.STRING:
-            return self._validate_string(value)
+            _value = self._get_string(value)
         elif t == Schema.INTEGER:
-            return self._validate_integer(value)
+            _value = self._get_integer(value)
         elif t == Schema.NUMBER:
-            return self._validate_number(value)
+            _value = self._get_number(value)
         elif t == Schema.MAP:
-            return self._validate_map(value)
+            _value = self._get_map(value, validate)
         elif t == Schema.LIST:
-            return self._validate_list(value)
+            _value = self._get_list(value, validate)
         elif t == Schema.BOOLEAN:
-            return self._validate_bool(value)
+            _value = self._get_bool(value)
 
-    def validate_data(self, value):
-        value = self._validate_data_type(value)
-        self.schema.validate_constraints(value, self.context)
-        return value
+        # property value resolves to None if resource it depends on is not
+        # created. So, if value is None skip constraint validation.
+        if value is not None and validate:
+            self.schema.validate_constraints(_value, self.context)
+
+        return _value
 
 
 class Properties(collections.Mapping):
@@ -307,11 +347,21 @@ class Properties(collections.Mapping):
 
     def validate(self, with_value=True):
         for (key, prop) in self.props.items():
+            # check that update_allowed and immutable
+            # do not contradict each other
+            if prop.update_allowed() and prop.immutable():
+                msg = _("Property %(prop)s: %(ua)s and %(im)s "
+                        "cannot both be True") % {
+                            'prop': key,
+                            'ua': prop.schema.UPDATE_ALLOWED,
+                            'im': prop.schema.IMMUTABLE}
+                raise exception.InvalidSchemaError(message=msg)
+
             if with_value:
                 try:
-                    self[key]
+                    self._get_property_value(key, validate=True)
                 except ValueError as e:
-                    msg = _("Property error : %s") % str(e)
+                    msg = _("Property error : %s") % e
                     raise exception.StackValidationFailed(message=msg)
 
             # are there unimplemented Properties
@@ -324,7 +374,7 @@ class Properties(collections.Mapping):
                 msg = _("Unknown Property %s") % key
                 raise exception.StackValidationFailed(message=msg)
 
-    def __getitem__(self, key):
+    def _get_property_value(self, key, validate=False):
         if key not in self:
             raise KeyError(_('%(prefix)sInvalid Property %(key)s') %
                            {'prefix': self.error_prefix, 'key': key})
@@ -333,18 +383,28 @@ class Properties(collections.Mapping):
 
         if key in self.data:
             try:
-                value = self.resolve(self.data[key])
-                return prop.validate_data(value)
+                unresolved_value = self.data[key]
+                if validate:
+                    deps = function.dependencies(unresolved_value)
+                    if any(res.action == res.INIT for res in deps):
+                        validate = False
+
+                value = self.resolve(unresolved_value)
+                return prop.get_value(value, validate)
             # the resolver function could raise any number of exceptions,
             # so handle this generically
             except Exception as e:
-                raise ValueError('%s%s %s' % (self.error_prefix, key, str(e)))
+                raise ValueError('%s%s %s' % (self.error_prefix, key,
+                                              six.text_type(e)))
         elif prop.has_default():
-            return prop.default()
+            return prop.get_value(None, validate)
         elif prop.required():
             raise ValueError(_('%(prefix)sProperty %(key)s not assigned') %
                              {'prefix': self.error_prefix, 'key': key})
 
+    def __getitem__(self, key):
+        return self._get_property_value(key)
+
     def __len__(self):
         return len(self.props)
 
@@ -363,7 +423,7 @@ class Properties(collections.Mapping):
             schema.INTEGER: parameters.Schema.NUMBER,
             schema.STRING: parameters.Schema.STRING,
             schema.NUMBER: parameters.Schema.NUMBER,
-            schema.BOOLEAN: parameters.Schema.STRING,
+            schema.BOOLEAN: parameters.Schema.BOOLEAN,
             schema.MAP: parameters.Schema.MAP,
             schema.LIST: parameters.Schema.LIST,
         }
@@ -411,8 +471,7 @@ class Properties(collections.Mapping):
 
     @classmethod
     def schema_to_parameters_and_properties(cls, schema):
-
-        '''Generates properties with params resolved for a resource's
+        """Generates properties with params resolved for a resource's
         properties_schema.
 
         :param schema: A resource type's properties_schema
@@ -427,15 +486,19 @@ class Properties(collections.Mapping):
                      'bar': {'Type': 'Json'}},
                     {'foo': {'Fn::Split': {'Ref': 'foo'}},
                      'bar': {'Ref': 'bar'}}
-        '''
+
+        """
         def param_prop_def_items(name, schema):
             param_def = cls._param_def_from_prop(schema)
             prop_def = cls._prop_def_from_prop(name, schema)
 
             return (name, param_def), (name, prop_def)
 
+        if not schema:
+            return {}, {}
+
         param_prop_defs = [param_prop_def_items(n, s)
-                           for n, s in schemata(schema).iteritems()
+                           for n, s in six.iteritems(schemata(schema))
                            if s.implemented]
         param_items, prop_items = zip(*param_prop_defs)
         return dict(param_items), dict(prop_items)
