@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -12,14 +11,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from heat.common import exception
-from heat.engine import clients
+from heat.engine import attributes
 from heat.engine import properties
 from heat.engine import resource
 from heat.engine.resources.vpc import VPC
-from heat.openstack.common import log as logging
-
-logger = logging.getLogger(__name__)
 
 
 class Subnet(resource.Resource):
@@ -36,6 +31,10 @@ class Subnet(resource.Resource):
         'Key', 'Value',
     )
 
+    ATTRIBUTES = (
+        AVAILABILITY_ZONE,
+    )
+
     properties_schema = {
         AVAILABILITY_ZONE: properties.Schema(
             properties.Schema.STRING,
@@ -72,6 +71,14 @@ class Subnet(resource.Resource):
         ),
     }
 
+    attributes_schema = {
+        AVAILABILITY_ZONE: attributes.Schema(
+            _('Availability Zone of the subnet.')
+        ),
+    }
+
+    default_client_name = 'neutron'
+
     def handle_create(self):
         client = self.neutron()
         # TODO(sbaker) Verify that this CidrBlock is within the vpc CidrBlock
@@ -84,16 +91,17 @@ class Subnet(resource.Resource):
             'ip_version': 4
         }
         subnet = client.create_subnet({'subnet': props})['subnet']
+        self.resource_id_set(subnet['id'])
 
         router = VPC.router_for_vpc(self.neutron(), network_id)
         if router:
             client.add_interface_router(
                 router['id'],
                 {'subnet_id': subnet['id']})
-        self.resource_id_set(subnet['id'])
 
     def handle_delete(self):
-        from neutronclient.common.exceptions import NeutronClientException
+        if self.resource_id is None:
+            return
 
         client = self.neutron()
         network_id = self.properties.get(self.VPC_ID)
@@ -105,26 +113,20 @@ class Subnet(resource.Resource):
                 client.remove_interface_router(
                     router['id'],
                     {'subnet_id': subnet_id})
-        except NeutronClientException as ex:
-            if ex.status_code != 404:
-                raise ex
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
         try:
             client.delete_subnet(subnet_id)
-        except NeutronClientException as ex:
-            if ex.status_code != 404:
-                raise ex
+        except Exception as ex:
+            self.client_plugin().ignore_not_found(ex)
 
-    def FnGetAtt(self, key):
-        if key == 'AvailabilityZone':
-            return self.properties.get(key)
-        raise exception.InvalidTemplateAttribute(resource=self.name, key=key)
+    def _resolve_attribute(self, name):
+        if name == self.AVAILABILITY_ZONE:
+            return self.properties.get(self.AVAILABILITY_ZONE)
 
 
 def resource_mapping():
-    if clients.neutronclient is None:
-        return {}
-
     return {
         'AWS::EC2::Subnet': Subnet,
     }
