@@ -1,4 +1,3 @@
-
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -18,6 +17,7 @@ import sys
 import fixtures
 import mock
 from oslo.config import cfg
+import six
 
 from heat.common import environment_format
 from heat.engine import environment
@@ -47,6 +47,59 @@ class EnvironmentTest(common.HeatTestCase):
         env = environment.Environment(new_env)
         self.assertEqual(new_env, env.user_env_as_dict())
 
+    def test_existing_parameters(self):
+        # This tests reusing the existing parameters as is
+        prev_params = {'foo': 'bar', 'tester': 'Yes'}
+        params = {}
+        expected = {'parameters': prev_params,
+                    'resource_registry': {'resources': {}}}
+        prev_env = environment.Environment(
+            {'parameters': prev_params,
+             'resource_registry': {'resources': {}}})
+        env = environment.Environment(params)
+        env.patch_previous_parameters(prev_env)
+        self.assertEqual(expected, env.user_env_as_dict())
+
+    def test_patch_existing_parameters(self):
+        # This tests patching cli parameters over the existing parameters
+        prev_params = {'foo': 'bar', 'tester': 'Yes'}
+        params = {'tester': 'patched'}
+        expected = {'parameters': {'foo': 'bar', 'tester': 'patched'},
+                    'resource_registry': {'resources': {}}}
+        prev_env = environment.Environment(
+            {'parameters': prev_params,
+             'resource_registry': {'resources': {}}})
+        env = environment.Environment(params)
+        env.patch_previous_parameters(prev_env)
+        self.assertEqual(expected, env.user_env_as_dict())
+
+    def test_patch_and_clear_existing_parameters(self):
+        # This tests patching cli parameters over the existing parameters
+        prev_params = {'foo': 'bar', 'tester': 'Yes',
+                       'another_tester': 'Yes'}
+        params = {'tester': 'patched'}
+        expected = {'parameters': {'foo': 'bar', 'tester': 'patched'},
+                    'resource_registry': {'resources': {}}}
+        prev_env = environment.Environment(
+            {'parameters': prev_params,
+             'resource_registry': {'resources': {}}})
+        env = environment.Environment(params)
+        env.patch_previous_parameters(prev_env, ['another_tester'])
+        self.assertEqual(expected, env.user_env_as_dict())
+
+    def test_clear_existing_parameters(self):
+        # This tests removing some parameters in the existing set of parameters
+        prev_params = {'foo': 'bar', 'tester': 'Yes'}
+        params = {}
+        expected = {'parameters': {'foo': 'bar'},
+                    'resource_registry': {'resources': {}}}
+        prev_env = environment.Environment(
+            {'parameters': prev_params,
+             'resource_registry': {'resources': {}}})
+        env = environment.Environment(params)
+        env.patch_previous_parameters(prev_env, ['tester'])
+        self.assertEqual(expected, env.user_env_as_dict())
+
     def test_global_registry(self):
         self.g_env.register_class('CloudX::Nova::Server',
                                   generic_resource.GenericResource)
@@ -152,7 +205,15 @@ def constraint_mapping():
         env = environment.Environment({})
         error = self.assertRaises(ValueError,
                                   resources._load_global_environment, env)
-        self.assertEqual("oops", str(error))
+        self.assertEqual("oops", six.text_type(error))
+
+    def test_constraints_registry_stevedore(self):
+        env = environment.Environment({})
+        resources._load_global_environment(env)
+
+        self.assertEqual("FlavorConstraint",
+                         env.get_constraint("nova.flavor").__name__)
+        self.assertIs(None, env.get_constraint("no_constraint"))
 
 
 class EnvironmentDuplicateTest(common.HeatTestCase):
@@ -178,7 +239,7 @@ class EnvironmentDuplicateTest(common.HeatTestCase):
         replace_log = 'Changing %s from %s to %s' % ('OS::Test::Dummy',
                                                      'test.yaml',
                                                      self.resource_type)
-        self.assertNotIn(replace_log, self.logger.output)
+        self.assertNotIn(replace_log, self.LOG.output)
         env_test = {u'resource_registry': {
             u'OS::Test::Dummy': self.resource_type}}
         env.load(env_test)
@@ -187,9 +248,9 @@ class EnvironmentDuplicateTest(common.HeatTestCase):
             # should return exactly the same object.
             self.assertIs(info, env.get_resource_info('OS::Test::Dummy',
                                                       'my_fip'))
-            self.assertNotIn(replace_log, self.logger.output)
+            self.assertNotIn(replace_log, self.LOG.output)
         else:
-            self.assertIn(replace_log, self.logger.output)
+            self.assertIn(replace_log, self.LOG.output)
             self.assertNotEqual(info,
                                 env.get_resource_info('OS::Test::Dummy',
                                                       'my_fip'))
