@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -12,22 +12,26 @@
 #    under the License.
 
 import json
+import mox
+import six
 import uuid
 
-from oslo.config import cfg
-
-cfg.CONF.import_opt('region_name_for_services', 'heat.common.config')
-cfg.CONF.import_group('keystone_authtoken',
-                      'keystoneclient.middleware.auth_token')
-
+from keystoneclient.auth.identity import v3 as ks_auth_v3
 import keystoneclient.exceptions as kc_exception
+from keystoneclient import session as ks_session
 from keystoneclient.v3 import client as kc_v3
+from keystoneclient.v3 import domains as kc_v3_domains
+from oslo.config import cfg
 
 from heat.common import exception
 from heat.common import heat_keystoneclient
 from heat.tests.common import HeatTestCase
 from heat.tests import utils
 
+cfg.CONF.import_opt('region_name_for_services', 'heat.common.config')
+cfg.CONF.import_group('keystone_authtoken',
+                      'keystonemiddleware.auth_token')
+
 
 class KeystoneClientTest(HeatTestCase):
     """Test cases for heat.common.heat_keystoneclient."""
@@ -37,6 +41,8 @@ class KeystoneClientTest(HeatTestCase):
 
         self.mock_admin_client = self.m.CreateMock(kc_v3.Client)
         self.mock_ks_v3_client = self.m.CreateMock(kc_v3.Client)
+        self.mock_ks_v3_client_domain_mngr = self.m.CreateMock(
+            kc_v3_domains.DomainManager)
         self.m.StubOutWithMock(kc_v3, "Client")
 
         dummy_url = 'http://server.test:5000/v2.0'
@@ -48,11 +54,14 @@ class KeystoneClientTest(HeatTestCase):
                               group='keystone_authtoken')
         cfg.CONF.set_override('admin_tenant_name', 'service',
                               group='keystone_authtoken')
-        cfg.CONF.set_override('stack_user_domain', 'adomain123')
+        cfg.CONF.set_override('stack_user_domain_id', 'adomain123')
         cfg.CONF.set_override('stack_domain_admin', 'adminuser123')
         cfg.CONF.set_override('stack_domain_admin_password', 'adminsecret')
         self.addCleanup(self.m.VerifyAll)
 
+    def _clear_domain_override(self):
+        cfg.CONF.clear_override('stack_user_domain_id')
+
     def _stub_admin_client(self, auth_ok=True):
         kc_v3.Client(
             auth_url='http://server.test:5000/v3',
@@ -64,6 +73,7 @@ class KeystoneClientTest(HeatTestCase):
             password='verybadpass',
             project_name='service',
             username='heat').AndReturn(self.mock_admin_client)
+        self.mock_admin_client.domains = self.mock_ks_v3_client_domain_mngr
         self.mock_admin_client.authenticate().AndReturn(auth_ok)
         if auth_ok:
             self.mock_admin_client.auth_ref = self.m.CreateMockAnything()
@@ -80,6 +90,7 @@ class KeystoneClientTest(HeatTestCase):
             password='adminsecret',
             user_domain_id='adomain123',
             username='adminuser123').AndReturn(self.mock_admin_client)
+        self.mock_admin_client.domains = self.mock_ks_v3_client_domain_mngr
         self.mock_admin_client.authenticate(
             domain_id='adomain123').AndReturn(auth_ok)
         if auth_ok:
@@ -87,7 +98,7 @@ class KeystoneClientTest(HeatTestCase):
             self.mock_admin_client.auth_ref.user_id = '1234'
 
     def _stubs_v3(self, method='token', auth_ok=True, trust_scoped=True,
-                  user_id='trustor_user_id'):
+                  user_id='trustor_user_id', auth_ref=None):
 
         if method == 'token':
             kc_v3.Client(
@@ -98,6 +109,15 @@ class KeystoneClientTest(HeatTestCase):
                 cert=None,
                 insecure=False,
                 key=None).AndReturn(self.mock_ks_v3_client)
+        elif method == 'auth_ref':
+            kc_v3.Client(
+                auth_ref=auth_ref,
+                auth_url='http://server.test:5000/v3',
+                endpoint='http://server.test:5000/v3',
+                cacert=None,
+                cert=None,
+                insecure=False,
+                key=None).AndReturn(self.mock_ks_v3_client)
         elif method == 'password':
             kc_v3.Client(
                 username='test_username',
@@ -125,7 +145,8 @@ class KeystoneClientTest(HeatTestCase):
             self.mock_ks_v3_client.auth_ref.trust_scoped = trust_scoped
             self.mock_ks_v3_client.auth_ref.auth_token = 'atrusttoken'
 
-        self.mock_ks_v3_client.authenticate().AndReturn(auth_ok)
+        if method != 'auth_ref':
+            self.mock_ks_v3_client.authenticate().AndReturn(auth_ok)
 
     def test_username_length(self):
         """Test that user names >64 characters are properly truncated."""
@@ -150,7 +171,8 @@ class KeystoneClientTest(HeatTestCase):
                                             ).AndReturn(mock_user)
 
         self.mock_ks_v3_client.roles = self.m.CreateMockAnything()
-        self.mock_ks_v3_client.roles.list().AndReturn(self._mock_roles_list())
+        self.mock_ks_v3_client.roles.list(
+            name='heat_stack_user').AndReturn(self._mock_roles_list())
         self.mock_ks_v3_client.roles.grant(project=ctx.tenant_id,
                                            role='4546',
                                            user='auser123').AndReturn(None)
@@ -171,22 +193,21 @@ class KeystoneClientTest(HeatTestCase):
         ctx.trust_id = None
 
         self.mock_ks_v3_client.roles = self.m.CreateMockAnything()
-        mock_roles_list = self._mock_roles_list(heat_stack_user='badrole')
-        self.mock_ks_v3_client.roles.list().AndReturn(mock_roles_list)
+        self.mock_ks_v3_client.roles.list(
+            name='heat_stack_user').AndReturn([])
         self.m.ReplayAll()
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         err = self.assertRaises(exception.Error,
                                 heat_ks_client.create_stack_user,
                                 'auser', password='password')
-        self.assertIn('Can\'t find role heat_stack_user', str(err))
+        self.assertIn('Can\'t find role heat_stack_user', six.text_type(err))
 
     def _mock_roles_list(self, heat_stack_user='heat_stack_user'):
         mock_roles_list = []
-        for r_id, r_name in (('1234', 'blah'), ('4546', heat_stack_user)):
-            mock_role = self.m.CreateMockAnything()
-            mock_role.id = r_id
-            mock_role.name = r_name
-            mock_roles_list.append(mock_role)
+        mock_role = self.m.CreateMockAnything()
+        mock_role.id = '4546'
+        mock_role.name = heat_stack_user
+        mock_roles_list.append(mock_role)
         return mock_roles_list
 
     def test_create_stack_domain_user(self):
@@ -206,7 +227,8 @@ class KeystoneClientTest(HeatTestCase):
                                             domain='adomain123'
                                             ).AndReturn(mock_user)
         self.mock_admin_client.roles = self.m.CreateMockAnything()
-        self.mock_admin_client.roles.list().AndReturn(self._mock_roles_list())
+        self.mock_admin_client.roles.list(
+            name='heat_stack_user').AndReturn(self._mock_roles_list())
         self.mock_admin_client.roles.grant(project='aproject',
                                            role='4546',
                                            user='duser123').AndReturn(None)
@@ -218,7 +240,7 @@ class KeystoneClientTest(HeatTestCase):
 
     def test_create_stack_domain_user_legacy_fallback(self):
         """Test creating a stack domain user, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -234,7 +256,8 @@ class KeystoneClientTest(HeatTestCase):
                                             ).AndReturn(mock_user)
 
         self.mock_ks_v3_client.roles = self.m.CreateMockAnything()
-        self.mock_ks_v3_client.roles.list().AndReturn(self._mock_roles_list())
+        self.mock_ks_v3_client.roles.list(
+            name='heat_stack_user').AndReturn(self._mock_roles_list())
         self.mock_ks_v3_client.roles.grant(project=ctx.tenant_id,
                                            role='4546',
                                            user='auser123').AndReturn(None)
@@ -254,15 +277,14 @@ class KeystoneClientTest(HeatTestCase):
 
         # mock keystone client functions
         self.mock_admin_client.roles = self.m.CreateMockAnything()
-        mock_roles_list = self._mock_roles_list(heat_stack_user='badrole')
-        self.mock_admin_client.roles.list().AndReturn(mock_roles_list)
+        self.mock_admin_client.roles.list(name='heat_stack_user').AndReturn([])
         self.m.ReplayAll()
 
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         err = self.assertRaises(exception.Error,
                                 heat_ks_client.create_stack_domain_user,
                                 username='duser', project_id='aproject')
-        self.assertIn('Can\'t find role heat_stack_user', str(err))
+        self.assertIn('Can\'t find role heat_stack_user', six.text_type(err))
 
     def test_delete_stack_domain_user(self):
         """Test deleting a stack domain user."""
@@ -293,7 +315,7 @@ class KeystoneClientTest(HeatTestCase):
 
     def test_delete_stack_domain_user_legacy_fallback(self):
         """Test deleting a stack domain user, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -388,6 +410,44 @@ class KeystoneClientTest(HeatTestCase):
         heat_ks_client.client
         self.assertIsNotNone(heat_ks_client._client)
 
+    def test_init_v3_token_auth_ref_v2(self):
+
+        """Test creating the client, token v2 auth_ref."""
+
+        expected_auth_ref = {'token': {'id': 'ctx_token', 'expires': '123'},
+                             'version': 'v2.0'}
+        self._stubs_v3(method='auth_ref', auth_ref=expected_auth_ref)
+        self.m.ReplayAll()
+
+        ctx = utils.dummy_context()
+        ctx.username = None
+        ctx.password = None
+        ctx.trust_id = None
+        ctx.auth_token = 'ctx_token'
+        ctx.auth_token_info = {'access': {'token': {'expires': '123'}}}
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        heat_ks_client.client
+        self.assertIsNotNone(heat_ks_client._client)
+
+    def test_init_v3_token_auth_ref_v3(self):
+
+        """Test creating the client, token v3 auth_ref."""
+
+        expected_auth_ref = {'auth_token': 'ctx_token',
+                             'expires': '456', 'version': 'v3'}
+        self._stubs_v3(method='auth_ref', auth_ref=expected_auth_ref)
+        self.m.ReplayAll()
+
+        ctx = utils.dummy_context()
+        ctx.username = None
+        ctx.password = None
+        ctx.trust_id = None
+        ctx.auth_token = 'ctx_token'
+        ctx.auth_token_info = {'token': {'expires': '456'}}
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        heat_ks_client.client
+        self.assertIsNotNone(heat_ks_client._client)
+
     def test_init_v3_password(self):
 
         """Test creating the client, password auth."""
@@ -401,6 +461,7 @@ class KeystoneClientTest(HeatTestCase):
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         client = heat_ks_client.client
         self.assertIsNotNone(client)
+        self.assertIsNone(ctx.trust_id)
 
     def test_init_v3_bad_nocreds(self):
 
@@ -431,7 +492,51 @@ class KeystoneClientTest(HeatTestCase):
         trust_context = heat_ks_client.create_trust_context()
         self.assertEqual(ctx.to_dict(), trust_context.to_dict())
 
-    def test_create_trust_context_trust_create(self):
+    def test_create_trust_context_trust_create_deletegate_subset_roles(self):
+        delegate_roles = ['heat_stack_owner']
+        self._test_create_trust_context_trust_create(delegate_roles)
+
+    def test_create_trust_context_trust_create_deletegate_all_roles(self):
+        delegate_roles = []
+        self._test_create_trust_context_trust_create(delegate_roles)
+
+    def _test_create_trust_context_trust_create(self, delegate_roles=None):
+
+        """Test create_trust_context when creating a trust."""
+
+        class MockTrust(object):
+            id = 'atrust123'
+
+        self._stub_admin_client()
+
+        self._stubs_v3()
+        cfg.CONF.set_override('deferred_auth_method', 'trusts')
+        cfg.CONF.set_override('trusts_delegated_roles', delegate_roles)
+
+        trustor_roles = ['heat_stack_owner', 'admin', '__member__']
+        trustee_roles = delegate_roles or trustor_roles
+        self.mock_ks_v3_client.auth_ref = self.m.CreateMockAnything()
+        self.mock_ks_v3_client.auth_ref.user_id = '5678'
+        self.mock_ks_v3_client.auth_ref.project_id = '42'
+        self.mock_ks_v3_client.trusts = self.m.CreateMockAnything()
+
+        self.mock_ks_v3_client.trusts.create(
+            trustor_user='5678',
+            trustee_user='1234',
+            project='42',
+            impersonation=True,
+            role_names=trustee_roles).AndReturn(MockTrust())
+
+        self.m.ReplayAll()
+
+        ctx = utils.dummy_context(roles=trustor_roles)
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        trust_context = heat_ks_client.create_trust_context()
+        self.assertEqual('atrust123', trust_context.trust_id)
+        self.assertEqual('5678', trust_context.trustor_user_id)
+
+    def test_create_trust_context_trust_create_norole(self):
 
         """Test create_trust_context when creating a trust."""
 
@@ -453,22 +558,23 @@ class KeystoneClientTest(HeatTestCase):
             trustee_user='1234',
             project='42',
             impersonation=True,
-            role_names=['heat_stack_owner']).AndReturn(MockTrust())
+            role_names=['heat_stack_owner']).AndRaise(kc_exception.NotFound())
 
         self.m.ReplayAll()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
-        trust_context = heat_ks_client.create_trust_context()
-        self.assertEqual('atrust123', trust_context.trust_id)
-        self.assertEqual('5678', trust_context.trustor_user_id)
+        exc = self.assertRaises(exception.MissingCredentialError,
+                                heat_ks_client.create_trust_context)
+        expected = 'Missing required credential: roles [\'heat_stack_owner\']'
+        self.assertIn(expected, six.text_type(exc))
 
     def test_init_domain_cfg_not_set_fallback(self):
 
         """Test error path when config lacks domain config."""
 
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
         cfg.CONF.clear_override('stack_domain_admin')
         cfg.CONF.clear_override('stack_domain_admin_password')
 
@@ -491,10 +597,11 @@ class KeystoneClientTest(HeatTestCase):
         ctx.trust_id = None
         err = self.assertRaises(exception.Error,
                                 heat_keystoneclient.KeystoneClient, ctx)
-        exp_msg = ('heat.conf misconfigured, cannot specify stack_user_domain '
-                   'without stack_domain_admin and '
-                   'stack_domain_admin_password')
-        self.assertIn(exp_msg, str(err))
+        exp_msg = ('heat.conf misconfigured, cannot specify '
+                   '"stack_user_domain_id" or "stack_user_domain_name" '
+                   'without "stack_domain_admin" and '
+                   '"stack_domain_admin_password"')
+        self.assertIn(exp_msg, six.text_type(err))
 
     def test_init_admin_client(self):
 
@@ -548,6 +655,7 @@ class KeystoneClientTest(HeatTestCase):
         ctx.trustor_user_id = 'trustor_user_id'
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         self.assertIsNotNone(heat_ks_client.client)
+        self.assertIsNone(ctx.auth_token)
 
     def test_trust_init_fail(self):
 
@@ -703,7 +811,7 @@ class KeystoneClientTest(HeatTestCase):
 
     def test_enable_stack_domain_user_legacy_fallback(self):
         """Test enabling a stack domain user, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -768,7 +876,7 @@ class KeystoneClientTest(HeatTestCase):
 
     def test_disable_stack_domain_user_legacy_fallback(self):
         """Test enabling a stack domain user, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -840,7 +948,7 @@ class KeystoneClientTest(HeatTestCase):
             credential_id='acredentialid')
 
     def test_delete_stack_domain_user_keypair_legacy_fallback(self):
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -887,8 +995,9 @@ class KeystoneClientTest(HeatTestCase):
                           user_id='duser123', project_id='aproject',
                           credential_id='acredentialid')
 
-    def _stub_uuid(self, values=[]):
+    def _stub_uuid(self, values=None):
         # stub UUID.hex to return the values specified
+        values = values or []
         self.m.StubOutWithMock(uuid, 'uuid4')
         for v in values:
             mock_uuid = self.m.CreateMockAnything()
@@ -969,7 +1078,7 @@ class KeystoneClientTest(HeatTestCase):
     def test_create_stack_domain_user_keypair_legacy_fallback(self):
 
         """Test creating ec2 credentials for domain user, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         self._stubs_v3()
 
@@ -1162,7 +1271,7 @@ class KeystoneClientTest(HeatTestCase):
 
     def test_create_stack_domain_project_legacy_fallback(self):
         """Test the create_stack_domain_project function, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1246,16 +1355,57 @@ class KeystoneClientTest(HeatTestCase):
 
         """Test the delete_stack_domain_project function."""
 
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
         heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
         heat_ks_client.delete_stack_domain_project(project_id='aprojectid')
 
+    def _stub_domain_user_pw_auth(self):
+        self.m.StubOutWithMock(ks_auth_v3, 'Password')
+        ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
+                            username='duser',
+                            password='apassw',
+                            project_id='aproject',
+                            user_domain_id='adomain123').AndReturn('dummyauth')
+
+    def test_stack_domain_user_token(self):
+        """Test stack_domain_user_token function."""
+        self._stub_domain_user_pw_auth()
+        dummysession = self.m.CreateMockAnything()
+        dummyresp = self.m.CreateMockAnything()
+        dummyresp.headers = {'X-Subject-Token': 'dummytoken'}
+        dummysession.post('http://server.test:5000/v3/auth/tokens?nocatalog',
+                          authenticated=False,
+                          headers={'Accept': 'application/json'},
+                          json=mox.IgnoreArg()).AndReturn(dummyresp)
+        self.m.StubOutWithMock(ks_session, 'Session')
+        ks_session.Session(auth='dummyauth').AndReturn(dummysession)
+        self.m.ReplayAll()
+
+        ctx = utils.dummy_context()
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        token = heat_ks_client.stack_domain_user_token(
+            username='duser', project_id='aproject', password='apassw')
+        self.assertEqual('dummytoken', token)
+
+    def test_stack_domain_user_token_err_nodomain(self):
+        """Test stack_domain_user_token error path."""
+        self._clear_domain_override()
+        ctx = utils.dummy_context()
+        ctx.trust_id = None
+        heat_ks_client = heat_keystoneclient.KeystoneClient(ctx)
+        self.assertRaises(exception.Error,
+                          heat_ks_client.stack_domain_user_token,
+                          username='user',
+                          project_id='aproject',
+                          password='password')
+
     def test_delete_stack_domain_project_legacy_fallback(self):
         """Test the delete_stack_domain_project function, fallback path."""
-        cfg.CONF.clear_override('stack_user_domain')
+        self._clear_domain_override()
 
         ctx = utils.dummy_context()
         ctx.trust_id = None
@@ -1316,3 +1466,131 @@ class KeystoneClientTest(HeatTestCase):
         }
         service_url = 'http://regionone.example.com:1234/v1'
         self._test_url_for(service_url, kwargs)
+
+
+class KeystoneClientTestDomainName(KeystoneClientTest):
+    def setUp(self):
+        cfg.CONF.set_override('stack_user_domain_name', 'fake_domain_name')
+        super(KeystoneClientTestDomainName, self).setUp()
+        cfg.CONF.clear_override('stack_user_domain_id')
+
+    def _clear_domain_override(self):
+        cfg.CONF.clear_override('stack_user_domain_name')
+
+    def _stub_domain_admin_client_domain_get(self):
+        dummy_domain = self.m.CreateMockAnything()
+        dummy_domain.id = 'adomain123'
+        self.mock_ks_v3_client_domain_mngr.list(
+            name='fake_domain_name').AndReturn([dummy_domain])
+
+    def _stub_domain_admin_client(self, auth_ok=True):
+        kc_v3.Client(
+            auth_url='http://server.test:5000/v3',
+            cacert=None,
+            cert=None,
+            endpoint='http://server.test:5000/v3',
+            insecure=False,
+            key=None,
+            password='adminsecret',
+            user_domain_name='fake_domain_name',
+            username='adminuser123').AndReturn(self.mock_admin_client)
+        self.mock_admin_client.domains = self.mock_ks_v3_client_domain_mngr
+        self.mock_admin_client.authenticate(
+            domain_name='fake_domain_name').AndReturn(auth_ok)
+        if auth_ok:
+            self.mock_admin_client.auth_ref = self.m.CreateMockAnything()
+            self.mock_admin_client.auth_ref.user_id = '1234'
+
+    def _stub_domain_user_pw_auth(self):
+        self.m.StubOutWithMock(ks_auth_v3, 'Password')
+        ks_auth_v3.Password(auth_url='http://server.test:5000/v3',
+                            username='duser',
+                            password='apassw',
+                            project_id='aproject',
+                            user_domain_name='fake_domain_name'
+                            ).AndReturn('dummyauth')
+
+    def test_enable_stack_domain_user_error_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_enable_stack_domain_user_error_project()
+
+    def test_delete_stack_domain_user_keypair(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user_keypair()
+
+    def test_delete_stack_domain_user_error_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user_error_project()
+
+    def test_delete_stack_domain_user_keypair_error_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user_keypair_error_project()
+
+    def test_delete_stack_domain_user(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user()
+
+    def test_enable_stack_domain_user(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_enable_stack_domain_user()
+
+    def test_delete_stack_domain_user_error_domain(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user_error_domain()
+
+    def test_disable_stack_domain_user_error_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_disable_stack_domain_user_error_project()
+
+    def test_enable_stack_domain_user_error_domain(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_enable_stack_domain_user_error_domain()
+
+    def test_delete_stack_domain_user_keypair_error_domain(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_user_keypair_error_domain()
+
+    def test_disable_stack_domain_user(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_disable_stack_domain_user()
+
+    def test_disable_stack_domain_user_error_domain(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_disable_stack_domain_user_error_domain()
+
+    def test_delete_stack_domain_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_project()
+
+    def test_delete_stack_domain_project_notfound(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_project_notfound()
+
+    def test_delete_stack_domain_project_wrongdomain(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_delete_stack_domain_project_wrongdomain()
+
+    def test_create_stack_domain_project(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_create_stack_domain_project()
+
+    def test_create_stack_domain_user(self):
+        self._stub_domain_admin_client_domain_get()
+        p = super(KeystoneClientTestDomainName, self)
+        p.test_create_stack_domain_user()
