@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,18 +11,22 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import copy
+import six
+
 from heat.common import exception
 from heat.common import identifier
 from heat.common import template_format
-from heat.engine import constraints
 from heat.engine import environment
 from heat.engine import function
+from heat.engine.hot import functions as hot_functions
 from heat.engine.hot import parameters as hot_param
 from heat.engine.hot import template as hot_template
 from heat.engine import parameters
 from heat.engine import parser
 from heat.engine import resource
 from heat.engine import resources
+from heat.engine import rsrc_defn
 from heat.engine import template
 from heat.tests.common import HeatTestCase
 from heat.tests import generic_resource as generic_rsrc
@@ -34,6 +38,17 @@ hot_tpl_empty = template_format.parse('''
 heat_template_version: 2013-05-23
 ''')
 
+hot_juno_tpl_empty = template_format.parse('''
+heat_template_version: 2014-10-16
+''')
+
+hot_tpl_empty_sections = template_format.parse('''
+heat_template_version: 2013-05-23
+parameters:
+resources:
+outputs:
+''')
+
 hot_tpl_generic_resource = template_format.parse('''
 heat_template_version: 2013-05-23
 resources:
@@ -62,6 +77,16 @@ resources:
 ''')
 
 
+class DummyClass(object):
+    metadata = None
+
+    def metadata_get(self):
+        return self.metadata
+
+    def metadata_set(self, metadata):
+        self.metadata = metadata
+
+
 class HOTemplateTest(HeatTestCase):
     """Test processing of HOT templates."""
 
@@ -74,7 +99,7 @@ class HOTemplateTest(HeatTestCase):
 
         tmpl = parser.Template(hot_tpl_empty)
         # check if we get the right class
-        self.assertIsInstance(tmpl, hot_template.HOTemplate)
+        self.assertIsInstance(tmpl, hot_template.HOTemplate20130523)
         # test getting an invalid section
         self.assertNotIn('foobar', tmpl)
 
@@ -83,6 +108,26 @@ class HOTemplateTest(HeatTestCase):
         self.assertEqual({}, tmpl[tmpl.RESOURCES])
         self.assertEqual({}, tmpl[tmpl.OUTPUTS])
 
+    def test_defaults_for_empty_sections(self):
+        """Test default secntion's content behavior of HOT template."""
+
+        tmpl = parser.Template(hot_tpl_empty_sections)
+        # check if we get the right class
+        self.assertIsInstance(tmpl, hot_template.HOTemplate20130523)
+        # test getting an invalid section
+        self.assertNotIn('foobar', tmpl)
+
+        # test defaults for valid sections
+        self.assertEqual('No description', tmpl[tmpl.DESCRIPTION])
+        self.assertEqual({}, tmpl[tmpl.RESOURCES])
+        self.assertEqual({}, tmpl[tmpl.OUTPUTS])
+
+        stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
+
+        self.assertIsNone(stack.parameters._validate_user_parameters())
+        self.assertIsNone(stack.parameters._validate_tmpl_parameters())
+        self.assertIsNone(stack.validate())
+
     def test_translate_resources_good(self):
         """Test translation of resources into internal engine format."""
 
@@ -133,7 +178,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Type" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_resources_bad_properties(self):
         """Test translation of resources including invalid keyword."""
@@ -157,7 +202,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Properties" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_resources_bad_metadata(self):
         """Test translation of resources including invalid keyword."""
@@ -181,7 +226,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"Metadata" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_resources_bad_depends_on(self):
         """Test translation of resources including invalid keyword."""
@@ -205,7 +250,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"DependsOn" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_resources_bad_deletion_polciy(self):
         """Test translation of resources including invalid keyword."""
@@ -229,7 +274,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"DeletionPolicy" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_resources_bad_update_policy(self):
         """Test translation of resources including invalid keyword."""
@@ -253,7 +298,7 @@ class HOTemplateTest(HeatTestCase):
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.RESOURCES)
         self.assertEqual('u\'"UpdatePolicy" is not a valid keyword '
                          'inside a resource definition\'',
-                         str(err))
+                         six.text_type(err))
 
     def test_translate_outputs_good(self):
         """Test translation of outputs into internal engine format."""
@@ -284,7 +329,7 @@ class HOTemplateTest(HeatTestCase):
 
         tmpl = parser.Template(hot_tpl)
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.OUTPUTS)
-        self.assertIn('Description', str(err))
+        self.assertIn('Description', six.text_type(err))
 
     def test_translate_outputs_bad_value(self):
         """Test translation of outputs into internal engine format."""
@@ -299,7 +344,7 @@ class HOTemplateTest(HeatTestCase):
 
         tmpl = parser.Template(hot_tpl)
         err = self.assertRaises(KeyError, tmpl.__getitem__, tmpl.OUTPUTS)
-        self.assertIn('Value', str(err))
+        self.assertIn('Value', six.text_type(err))
 
     def test_resource_group_list_join(self):
         """Test list_join on a ResourceGroup's inner attributes
@@ -308,7 +353,7 @@ class HOTemplateTest(HeatTestCase):
         can return the list of the runtime values.
         """
         hot_tpl = template_format.parse('''
-        heat_template_version: 2013-05-23
+        heat_template_version: 2014-10-16
         resources:
           rg:
             type: OS::Heat::ResourceGroup
@@ -319,7 +364,7 @@ class HOTemplateTest(HeatTestCase):
         ''')
         tmpl = parser.Template(hot_tpl)
         stack = parser.Stack(utils.dummy_context(), 'test_stack', tmpl)
-        snippet = {'Fn::Join': ["\n", {'get_attr': ['rg', 'name']}]}
+        snippet = {'list_join': ["\n", {'get_attr': ['rg', 'name']}]}
         self.assertEqual(self.resolve(snippet, tmpl, stack), '')
 
     def test_str_replace(self):
@@ -433,7 +478,7 @@ class HOTemplateTest(HeatTestCase):
                                       snippet, tmpl, stack)
         self.assertEqual(
             'Argument to "get_file" must be a string',
-            str(notStrErr))
+            six.text_type(notStrErr))
 
     def test_get_file_missing_files(self):
         """Test get_file function with no matching key in files section."""
@@ -450,7 +495,7 @@ class HOTemplateTest(HeatTestCase):
         self.assertEqual(
             ('No content found in the "files" section for '
              'get_file path: file:///tmp/foo.yaml'),
-            str(missingErr))
+            six.text_type(missingErr))
 
     def test_get_file_nested_does_not_resolve(self):
         """Test get_file function does not resolve nested calls."""
@@ -467,7 +512,7 @@ class HOTemplateTest(HeatTestCase):
 
     def test_prevent_parameters_access(self):
         """
-        Test that the parameters section can't be accesed using the template
+        Test that the parameters section can't be accessed using the template
         as a dictionary.
         """
         expected_description = "This can be accessed"
@@ -486,11 +531,11 @@ class HOTemplateTest(HeatTestCase):
 
         #Hot template test
         keyError = self.assertRaises(KeyError, tmpl.__getitem__, 'parameters')
-        self.assertIn(err_str, str(keyError))
+        self.assertIn(err_str, six.text_type(keyError))
 
         #CFN template test
         keyError = self.assertRaises(KeyError, tmpl.__getitem__, 'Parameters')
-        self.assertIn(err_str, str(keyError))
+        self.assertIn(err_str, six.text_type(keyError))
 
     def test_parameters_section_not_iterable(self):
         """
@@ -538,15 +583,15 @@ class HOTemplateTest(HeatTestCase):
         deletion_policy_snippet = {'resource_facade': 'deletion_policy'}
         update_policy_snippet = {'resource_facade': 'update_policy'}
 
-        class DummyClass(object):
-            pass
         parent_resource = DummyClass()
-        parent_resource.metadata = {"foo": "bar"}
-        parent_resource.t = {'DeletionPolicy': 'Retain',
-                             'UpdatePolicy': {"blarg": "wibble"}}
+        parent_resource.metadata_set({"foo": "bar"})
+        parent_resource.t = rsrc_defn.ResourceDefinition(
+            'parent', 'SomeType',
+            deletion_policy=rsrc_defn.ResourceDefinition.RETAIN,
+            update_policy={"blarg": "wibble"})
         parent_resource.stack = parser.Stack(utils.dummy_context(),
                                              'toplevel_stack',
-                                             parser.Template({}))
+                                             parser.Template(hot_tpl_empty))
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
                              parent_resource=parent_resource)
@@ -560,18 +605,17 @@ class HOTemplateTest(HeatTestCase):
     def test_resource_facade_function(self):
         deletion_policy_snippet = {'resource_facade': 'deletion_policy'}
 
-        class DummyClass(object):
-            pass
         parent_resource = DummyClass()
-        parent_resource.metadata = {"foo": "bar"}
+        parent_resource.metadata_set({"foo": "bar"})
+        tmpl = parser.Template(hot_juno_tpl_empty)
         parent_resource.stack = parser.Stack(utils.dummy_context(),
                                              'toplevel_stack',
-                                             parser.Template({}))
-        parent_snippet = {'DeletionPolicy': {'Fn::Join': ['eta',
-                                                          ['R', 'in']]}}
-        parent_tmpl = parent_resource.stack.t.parse(parent_resource.stack,
-                                                    parent_snippet)
-        parent_resource.t = parent_tmpl
+                                             tmpl)
+        del_policy = hot_functions.Join(parent_resource.stack,
+                                        'list_join', ['eta', ['R', 'in']])
+        parent_resource.t = rsrc_defn.ResourceDefinition(
+            'parent', 'SomeType',
+            deletion_policy=del_policy)
 
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
@@ -587,24 +631,56 @@ class HOTemplateTest(HeatTestCase):
                                   self.resolve,
                                   snippet,
                                   stack.t, stack)
-        self.assertIn(snippet.keys()[0], str(error))
+        self.assertIn(snippet.keys()[0], six.text_type(error))
 
     def test_resource_facade_missing_deletion_policy(self):
         snippet = {'resource_facade': 'deletion_policy'}
 
-        class DummyClass(object):
-            pass
         parent_resource = DummyClass()
-        parent_resource.metadata = {"foo": "bar"}
-        parent_resource.t = {}
+        parent_resource.metadata_set({"foo": "bar"})
+        parent_resource.t = rsrc_defn.ResourceDefinition('parent', 'SomeType')
         parent_resource.stack = parser.Stack(utils.dummy_context(),
                                              'toplevel_stack',
-                                             parser.Template({}))
+                                             parser.Template(hot_tpl_empty))
         stack = parser.Stack(utils.dummy_context(), 'test_stack',
                              parser.Template(hot_tpl_empty),
                              parent_resource=parent_resource)
         self.assertEqual('Delete', self.resolve(snippet, stack.t, stack))
 
+    def test_removed_function(self):
+        snippet = {'Fn::GetAZs': ''}
+        stack = parser.Stack(utils.dummy_context(), 'test_stack',
+                             parser.Template(hot_juno_tpl_empty))
+        error = self.assertRaises(exception.InvalidTemplateVersion,
+                                  function.validate,
+                                  stack.t.parse(stack, snippet))
+        self.assertIn(snippet.keys()[0], six.text_type(error))
+
+    def test_add_resource(self):
+        hot_tpl = template_format.parse('''
+        heat_template_version: 2013-05-23
+        resources:
+          resource1:
+            type: AWS::EC2::Instance
+            properties:
+              property1: value1
+            metadata:
+              foo: bar
+            depends_on:
+              - dummy
+            deletion_policy: Retain
+            update_policy:
+              foo: bar
+        ''')
+        source = parser.Template(hot_tpl)
+        empty = parser.Template(copy.deepcopy(hot_tpl_empty))
+        stack = parser.Stack(utils.dummy_context(), 'test_stack', source)
+
+        for defn in source.resource_definitions(stack).values():
+            empty.add_resource(defn)
+
+        self.assertEqual(hot_tpl['resources'], empty.t['resources'])
+
 
 class StackTest(test_parser.StackTest):
     """Test stack function when stack was created from HOT template."""
@@ -612,7 +688,6 @@ class StackTest(test_parser.StackTest):
     def resolve(self, snippet):
         return function.resolve(self.stack.t.parse(self.stack, snippet))
 
-    @utils.stack_delete_after
     def test_get_attr_multiple_rsrc_status(self):
         """Test resolution of get_attr occurrences in HOT template."""
 
@@ -639,7 +714,6 @@ class StackTest(test_parser.StackTest):
             # resource name.
             self.assertEqual({'Value': 'resource1'}, self.resolve(snippet))
 
-    @utils.stack_delete_after
     def test_get_attr_invalid(self):
         """Test resolution of get_attr occurrences in HOT template."""
 
@@ -654,7 +728,6 @@ class StackTest(test_parser.StackTest):
                           self.resolve,
                           {'Value': {'get_attr': ['resource1', 'NotThere']}})
 
-    @utils.stack_delete_after
     def test_get_attr_invalid_resource(self):
         """Test resolution of get_attr occurrences in HOT template."""
 
@@ -668,10 +741,9 @@ class StackTest(test_parser.StackTest):
                          self.stack.state)
 
         snippet = {'Value': {'get_attr': ['resource2', 'who_cares']}}
-        self.assertRaises(exception.InvalidTemplateAttribute,
+        self.assertRaises(exception.InvalidTemplateReference,
                           self.resolve, snippet)
 
-    @utils.stack_delete_after
     def test_get_resource(self):
         """Test resolution of get_resource occurrences in HOT template."""
 
@@ -686,7 +758,6 @@ class StackTest(test_parser.StackTest):
         snippet = {'value': {'get_resource': 'resource1'}}
         self.assertEqual({'value': 'resource1'}, self.resolve(snippet))
 
-    @utils.stack_delete_after
     def test_set_param_id(self):
         tmpl = parser.Template(hot_tpl_empty)
         self.stack = parser.Stack(self.ctx, 'param_id_test', tmpl)
@@ -705,7 +776,6 @@ class StackTest(test_parser.StackTest):
         self.assertFalse(params.set_stack_id(None))
         self.assertTrue(params.set_stack_id(stack_id))
 
-    @utils.stack_delete_after
     def test_set_param_id_update(self):
         tmpl = template.Template(
             {'heat_template_version': '2013-05-23',
@@ -732,9 +802,9 @@ class StackTest(test_parser.StackTest):
                          (parser.Stack.UPDATE, parser.Stack.COMPLETE))
         self.assertEqual(self.stack['AResource'].properties['Foo'], 'xyz')
 
-        self.assertEqual(self.stack['AResource'].metadata['Bar'], stack_id)
+        self.assertEqual(
+            self.stack['AResource'].metadata_get()['Bar'], stack_id)
 
-    @utils.stack_delete_after
     def test_load_param_id(self):
         tmpl = parser.Template(hot_tpl_empty)
         self.stack = parser.Stack(self.ctx, 'param_load_id_test', tmpl)
@@ -747,7 +817,6 @@ class StackTest(test_parser.StackTest):
         self.assertEqual(newstack.parameters['OS::stack_id'],
                          stack_identifier.stack_id)
 
-    @utils.stack_delete_after
     def test_update_modify_param_ok_replace(self):
         tmpl = {
             'heat_template_version': '2013-05-23',
@@ -802,7 +871,6 @@ class StackAttributesTest(HeatTestCase):
     def setUp(self):
         super(StackAttributesTest, self).setUp()
 
-        utils.setup_dummy_db()
         self.ctx = utils.dummy_context()
 
         resource._register_class('GenericResourceType',
@@ -868,7 +936,6 @@ class StackAttributesTest(HeatTestCase):
               expected={'Value': None}))
     ]
 
-    @utils.stack_delete_after
     def test_get_attr(self):
         """Test resolution of get_attr occurrences in HOT template."""
 
@@ -914,7 +981,7 @@ class StackGetAttrValidationTest(HeatTestCase):
         try:
             stack.validate()
         except exception.StackValidationFailed as exc:
-            self.fail("Validation should have passed: %s" % str(exc))
+            self.fail("Validation should have passed: %s" % six.text_type(exc))
         self.assertEqual([],
                          stack.resources['resource2'].properties['a_list'])
         self.assertEqual({},
@@ -1035,26 +1102,26 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['db_name']
 
         def v(value):
-            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema = hot_param.HOTParamSchema.from_dict(name, schema)
             param_schema.validate()
-            param_schema.validate_value(name, value)
+            param_schema.validate_value(value)
             return True
 
         value = 'wp'
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(len_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(len_desc, six.text_type(err))
 
         value = 'abcdefghijklmnopq'
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(len_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(len_desc, six.text_type(err))
 
         value = 'abcdefgh1'
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(pattern_desc1, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(pattern_desc1, six.text_type(err))
 
         value = 'Abcdefghi'
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(pattern_desc2, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(pattern_desc2, six.text_type(err))
 
         value = 'abcdefghi'
         self.assertTrue(v(value))
@@ -1090,20 +1157,24 @@ class HOTParamValidatorTest(HeatTestCase):
             return True
 
         value = 'wp'
-        err = self.assertRaises(ValueError, run_parameters, value)
-        self.assertIn(len_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed,
+                                run_parameters, value)
+        self.assertIn(len_desc, six.text_type(err))
 
         value = 'abcdefghijklmnopq'
-        err = self.assertRaises(ValueError, run_parameters, value)
-        self.assertIn(len_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed,
+                                run_parameters, value)
+        self.assertIn(len_desc, six.text_type(err))
 
         value = 'abcdefgh1'
-        err = self.assertRaises(ValueError, run_parameters, value)
-        self.assertIn(pattern_desc1, str(err))
+        err = self.assertRaises(exception.StackValidationFailed,
+                                run_parameters, value)
+        self.assertIn(pattern_desc1, six.text_type(err))
 
         value = 'Abcdefghi'
-        err = self.assertRaises(ValueError, run_parameters, value)
-        self.assertIn(pattern_desc2, str(err))
+        err = self.assertRaises(exception.StackValidationFailed,
+                                run_parameters, value)
+        self.assertIn(pattern_desc2, six.text_type(err))
 
         value = 'abcdefghi'
         self.assertTrue(run_parameters(value))
@@ -1126,18 +1197,18 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['db_port']
 
         def v(value):
-            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema = hot_param.HOTParamSchema.from_dict(name, schema)
             param_schema.validate()
-            param_schema.validate_value(name, value)
+            param_schema.validate_value(value)
             return True
 
         value = 29999
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(range_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(range_desc, six.text_type(err))
 
         value = 50001
-        err = self.assertRaises(ValueError, v, value)
-        self.assertIn(range_desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertIn(range_desc, six.text_type(err))
 
         value = 30000
         self.assertTrue(v(value))
@@ -1169,18 +1240,18 @@ class HOTParamValidatorTest(HeatTestCase):
         schema = param['param1']
 
         def v(value):
-            param_schema = hot_param.HOTParamSchema.from_dict(schema)
+            param_schema = hot_param.HOTParamSchema.from_dict(name, schema)
             param_schema.validate()
-            param_schema.validate_value(name, value)
+            param_schema.validate_value(value)
             return True
 
         value = "1"
-        err = self.assertRaises(ValueError, v, value)
-        self.assertEqual(desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertEqual(desc, six.text_type(err))
 
         value = "2"
-        err = self.assertRaises(ValueError, v, value)
-        self.assertEqual(desc, str(err))
+        err = self.assertRaises(exception.StackValidationFailed, v, value)
+        self.assertEqual(desc, six.text_type(err))
 
         value = "0"
         self.assertTrue(v(value))
@@ -1196,10 +1267,11 @@ class HOTParamValidatorTest(HeatTestCase):
                     {'range': {'min': 30000, 'max': 50000},
                      'description': range_desc}]}}
 
-        schema = hot_param.HOTParamSchema.from_dict(param['db_port'])
-        err = self.assertRaises(constraints.InvalidSchemaError,
+        schema = hot_param.HOTParamSchema.from_dict('db_port',
+                                                    param['db_port'])
+        err = self.assertRaises(exception.InvalidSchemaError,
                                 schema.validate)
-        self.assertIn(range_desc, str(err))
+        self.assertIn(range_desc, six.text_type(err))
 
     def test_validate_schema_wrong_key(self):
         hot_tpl = template_format.parse('''
@@ -1209,9 +1281,10 @@ class HOTParamValidatorTest(HeatTestCase):
                 foo: bar
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
-        self.assertEqual("Invalid key 'foo' for parameter", str(error))
+        self.assertEqual("Invalid key 'foo' for parameter (param1)",
+                         six.text_type(error))
 
     def test_validate_schema_no_type(self):
         hot_tpl = template_format.parse('''
@@ -1221,9 +1294,10 @@ class HOTParamValidatorTest(HeatTestCase):
                 description: Hi!
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
-        self.assertEqual("Missing parameter type", str(error))
+        self.assertEqual("Missing parameter type for parameter: param1",
+                         six.text_type(error))
 
     def test_validate_schema_unknown_type(self):
         hot_tpl = template_format.parse('''
@@ -1233,10 +1307,10 @@ class HOTParamValidatorTest(HeatTestCase):
                 type: Unicode
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid type (Unicode)", str(error))
+            "Invalid type (Unicode)", six.text_type(error))
 
     def test_validate_schema_constraints(self):
         hot_tpl = template_format.parse('''
@@ -1249,11 +1323,11 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
             "Invalid key 'allowed_valus' for parameter constraints",
-            str(error))
+            six.text_type(error))
 
     def test_validate_schema_constraints_not_list(self):
         hot_tpl = template_format.parse('''
@@ -1265,10 +1339,11 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid parameter constraints, expected a list", str(error))
+            "Invalid parameter constraints for parameter param1, "
+            "expected a list", six.text_type(error))
 
     def test_validate_schema_constraints_not_mapping(self):
         hot_tpl = template_format.parse('''
@@ -1280,10 +1355,11 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid parameter constraints, expected a mapping", str(error))
+            "Invalid parameter constraints, expected a mapping",
+            six.text_type(error))
 
     def test_validate_schema_empty_constraints(self):
         hot_tpl = template_format.parse('''
@@ -1296,9 +1372,9 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
-        self.assertEqual("No constraint expressed", str(error))
+        self.assertEqual("No constraint expressed", six.text_type(error))
 
     def test_validate_schema_constraints_range_wrong_format(self):
         hot_tpl = template_format.parse('''
@@ -1311,10 +1387,11 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid range constraint, expected a mapping", str(error))
+            "Invalid range constraint, expected a mapping",
+            six.text_type(error))
 
     def test_validate_schema_constraints_range_invalid_key(self):
         hot_tpl = template_format.parse('''
@@ -1327,10 +1404,10 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: 1
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid key 'foo' for range constraint", str(error))
+            "Invalid key 'foo' for range constraint", six.text_type(error))
 
     def test_validate_schema_constraints_length_wrong_format(self):
         hot_tpl = template_format.parse('''
@@ -1343,10 +1420,11 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid length constraint, expected a mapping", str(error))
+            "Invalid length constraint, expected a mapping",
+            six.text_type(error))
 
     def test_validate_schema_constraints_length_invalid_key(self):
         hot_tpl = template_format.parse('''
@@ -1359,10 +1437,10 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "Invalid key 'foo' for length constraint", str(error))
+            "Invalid key 'foo' for length constraint", six.text_type(error))
 
     def test_validate_schema_constraints_wrong_allowed_pattern(self):
         hot_tpl = template_format.parse('''
@@ -1375,7 +1453,7 @@ class HOTParamValidatorTest(HeatTestCase):
                 default: foo
         ''')
         error = self.assertRaises(
-            constraints.InvalidSchemaError, parameters.Parameters,
+            exception.InvalidSchemaError, parameters.Parameters,
             "stack_testit", parser.Template(hot_tpl))
         self.assertEqual(
-            "AllowedPattern must be a string", str(error))
+            "AllowedPattern must be a string", six.text_type(error))
