@@ -1,4 +1,4 @@
-
+#
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
 #    a copy of the License at
@@ -11,6 +11,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import six
 import uuid
 
 import mock
@@ -23,12 +24,14 @@ from heat.engine import parser
 from heat.engine import resource
 from heat.engine import scheduler
 from heat.engine import stack_resource
+from heat.engine import template as templatem
 from heat.tests.common import HeatTestCase
 from heat.tests import generic_resource as generic_rsrc
 from heat.tests import utils
 
 
-ws_res_snippet = {"Type": "some_magic_type",
+ws_res_snippet = {"HeatTemplateFormatVersion": "2012-12-12",
+                  "Type": "some_magic_type",
                   "metadata": {
                       "key": "value",
                       "some": "more stuff"}}
@@ -98,20 +101,23 @@ class MyImplementedStackResource(MyStackResource):
 
 
 class StackResourceTest(HeatTestCase):
-
     def setUp(self):
         super(StackResourceTest, self).setUp()
-        utils.setup_dummy_db()
         resource._register_class('some_magic_type',
                                  MyStackResource)
         resource._register_class('GenericResource',
                                  generic_rsrc.GenericResource)
-        t = parser.Template({'Resources':
-                             {"provider_resource": ws_res_snippet}})
+        self.ws_resname = "provider_resource"
+        t = parser.Template({'HeatTemplateFormatVersion': '2012-12-12',
+                             'Resources':
+                             {self.ws_resname: ws_res_snippet}})
         self.parent_stack = parser.Stack(utils.dummy_context(), 'test_stack',
-                                         t, stack_id=str(uuid.uuid4()))
+                                         t, stack_id=str(uuid.uuid4()),
+                                         user_creds_id='uc123',
+                                         stack_user_project_id='aprojectid')
+        resource_defns = t.resource_definitions(self.parent_stack)
         self.parent_resource = MyStackResource('test',
-                                               ws_res_snippet,
+                                               resource_defns[self.ws_resname],
                                                self.parent_stack)
         self.templ = template_format.parse(param_template)
         self.simple_template = template_format.parse(simple_template)
@@ -128,6 +134,17 @@ class StackResourceTest(HeatTestCase):
         preview = self.parent_resource.preview()
         self.assertIsInstance(preview, stack_resource.StackResource)
 
+    def test_implementation_signature(self):
+        self.parent_resource.child_template = mock.Mock(
+            return_value=self.simple_template)
+        sig1, sig2 = self.parent_resource.implementation_signature()
+        self.assertEqual('3700dc2ae6ff4f0a236e7477ad6b8d51157f2153', sig1)
+        self.assertEqual('70e69545e0c47123159974c9166741085eb46dba', sig2)
+        self.parent_stack.t.files["foo"] = "bar"
+        sig1a, sig2a = self.parent_resource.implementation_signature()
+        self.assertEqual(sig1, sig1a)
+        self.assertNotEqual(sig2, sig2a)
+
     def test_propagated_files(self):
         self.parent_stack.t.files["foo"] = "bar"
         self.parent_resource.create_with_template(self.templ,
@@ -136,20 +153,20 @@ class StackResourceTest(HeatTestCase):
         self.assertEqual({"foo": "bar"}, self.stack.t.files)
 
     @mock.patch.object(stack_resource.StackResource, '_nested_environment')
-    @mock.patch.object(stack_resource.parser, 'Template')
     @mock.patch.object(stack_resource.parser, 'Stack')
     def test_preview_with_implemented_child_resource(self, mock_stack_class,
-                                                     mock_template_class,
                                                      mock_env_class):
         nested_stack = mock.Mock()
         mock_stack_class.return_value = nested_stack
         nested_stack.preview_resources.return_value = 'preview_nested_stack'
-        mock_template_class.return_value = 'parsed_template'
         mock_env_class.return_value = 'environment'
-        template = template_format.parse(param_template)
-        parent_resource = MyImplementedStackResource('test',
-                                                     ws_res_snippet,
-                                                     self.parent_stack)
+        template = templatem.Template(template_format.parse(param_template))
+        parent_t = self.parent_stack.t
+        resource_defns = parent_t.resource_definitions(self.parent_stack)
+        parent_resource = MyImplementedStackResource(
+            'test',
+            resource_defns[self.ws_resname],
+            self.parent_stack)
         params = {'KeyName': 'test'}
         parent_resource.set_template(template, params)
         validation_mock = mock.Mock(return_value=None)
@@ -157,35 +174,107 @@ class StackResourceTest(HeatTestCase):
 
         result = parent_resource.preview()
         mock_env_class.assert_called_once_with(params)
-        mock_template_class.assert_called_once_with(template)
         self.assertEqual('preview_nested_stack', result)
         mock_stack_class.assert_called_once_with(
             mock.ANY,
             'test_stack-test',
-            'parsed_template',
+            mock.ANY,
             'environment',
-            disable_rollback=mock.ANY,
+            timeout_mins=None,
+            disable_rollback=True,
             parent_resource=parent_resource,
-            owner_id=mock.ANY
+            owner_id=self.parent_stack.id,
+            user_creds_id=self.parent_stack.user_creds_id,
+            stack_user_project_id=self.parent_stack.stack_user_project_id,
+            adopt_stack_data=None,
         )
 
+    @mock.patch.object(stack_resource.StackResource, '_nested_environment')
+    @mock.patch.object(stack_resource.parser, 'Stack')
+    def test_preview_with_implemented_dict_child_resource(self,
+                                                          mock_stack_class,
+                                                          mock_env_class):
+        nested_stack = mock.Mock()
+        mock_stack_class.return_value = nested_stack
+        nested_stack.preview_resources.return_value = 'preview_nested_stack'
+        mock_env_class.return_value = 'environment'
+        template_dict = template_format.parse(param_template)
+        parent_t = self.parent_stack.t
+        resource_defns = parent_t.resource_definitions(self.parent_stack)
+        parent_resource = MyImplementedStackResource(
+            'test',
+            resource_defns[self.ws_resname],
+            self.parent_stack)
+        params = {'KeyName': 'test'}
+        parent_resource.set_template(template_dict, params)
+        validation_mock = mock.Mock(return_value=None)
+        parent_resource._validate_nested_resources = validation_mock
+
+        result = parent_resource.preview()
+        mock_env_class.assert_called_once_with(params)
+        self.assertEqual('preview_nested_stack', result)
+        mock_stack_class.assert_called_once_with(
+            mock.ANY,
+            'test_stack-test',
+            mock.ANY,
+            'environment',
+            timeout_mins=None,
+            disable_rollback=True,
+            parent_resource=parent_resource,
+            owner_id=self.parent_stack.id,
+            user_creds_id=self.parent_stack.user_creds_id,
+            stack_user_project_id=self.parent_stack.stack_user_project_id,
+            adopt_stack_data=None,
+        )
+
+    def test_preview_propagates_files(self):
+        self.parent_stack.t.files["foo"] = "bar"
+        tmpl = self.parent_stack.t.t
+        self.parent_resource.child_template = mock.Mock(return_value=tmpl)
+        self.parent_resource.child_params = mock.Mock(return_value={})
+        self.parent_resource.preview()
+        self.stack = self.parent_resource.nested()
+        self.assertEqual({"foo": "bar"}, self.stack.t.files)
+
     def test_preview_validates_nested_resources(self):
-        stack_resource = MyImplementedStackResource('test',
-                                                    ws_res_snippet,
-                                                    self.parent_stack)
-        stack_resource.child_template = mock.Mock(return_value={})
-        stack_resource.child_params = mock.Mock()
+        parent_t = self.parent_stack.t
+        resource_defns = parent_t.resource_definitions(self.parent_stack)
+        stk_resource = MyImplementedStackResource(
+            'test',
+            resource_defns[self.ws_resname],
+            self.parent_stack)
+        stk_resource.child_template = \
+            mock.Mock(return_value=templatem.Template(self.simple_template))
+        stk_resource.child_params = mock.Mock()
         exc = exception.RequestLimitExceeded(message='Validation Failed')
         validation_mock = mock.Mock(side_effect=exc)
-        stack_resource._validate_nested_resources = validation_mock
+        stk_resource._validate_nested_resources = validation_mock
 
         self.assertRaises(exception.RequestLimitExceeded,
-                          stack_resource.preview)
+                          stk_resource.preview)
+
+    def test_preview_dict_validates_nested_resources(self):
+        parent_t = self.parent_stack.t
+        resource_defns = parent_t.resource_definitions(self.parent_stack)
+        stk_resource = MyImplementedStackResource(
+            'test',
+            resource_defns[self.ws_resname],
+            self.parent_stack)
+        stk_resource.child_template = \
+            mock.Mock(return_value=self.simple_template)
+        stk_resource.child_params = mock.Mock()
+        exc = exception.RequestLimitExceeded(message='Validation Failed')
+        validation_mock = mock.Mock(side_effect=exc)
+        stk_resource._validate_nested_resources = validation_mock
+
+        self.assertRaises(exception.RequestLimitExceeded,
+                          stk_resource.preview)
 
     def test__validate_nested_resources_checks_num_of_resources(self):
         stack_resource.cfg.CONF.set_override('max_resources_per_stack', 2)
-        tmpl = {'Resources': [1]}
-        template = stack_resource.parser.Template(tmpl)
+        tmpl = {'HeatTemplateFormatVersion': '2012-12-12',
+                'Resources': [1]}
+        template = stack_resource.template.Template(tmpl)
         root_resources = mock.Mock(return_value=2)
         self.parent_resource.stack.root_stack.total_resources = root_resources
 
@@ -193,7 +282,6 @@ class StackResourceTest(HeatTestCase):
                           self.parent_resource._validate_nested_resources,
                           template)
 
-    @utils.stack_delete_after
     def test_create_with_template_ok(self):
         self.parent_resource.create_with_template(self.templ,
                                                   {"KeyName": "key"})
@@ -205,8 +293,8 @@ class StackResourceTest(HeatTestCase):
         self.assertEqual(self.templ, self.stack.t.t)
         self.assertEqual(self.stack.id, self.parent_resource.resource_id)
         self.assertIsNone(self.stack.timeout_mins)
+        self.assertEqual('aprojectid', self.stack.stack_user_project_id)
 
-    @utils.stack_delete_after
     def test_create_with_template_timeout_mins(self):
         self.assertIsNone(self.parent_stack.timeout_mins)
         self.m.StubOutWithMock(self.parent_stack, 'timeout_mins')
@@ -218,7 +306,6 @@ class StackResourceTest(HeatTestCase):
         self.assertEqual(100, self.stack.timeout_mins)
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_adopt_with_template_ok(self):
         adopt_data = {
             "resources": {
@@ -241,32 +328,24 @@ class StackResourceTest(HeatTestCase):
         self.assertEqual(self.templ, self.stack.t.t)
         self.assertEqual(self.stack.id, self.parent_resource.resource_id)
 
-    @utils.stack_delete_after
-    def test_set_deletion_policy(self):
+    def test_prepare_abandon(self):
         self.parent_resource.create_with_template(self.templ,
                                                   {"KeyName": "key"})
-        self.stack = self.parent_resource.nested()
-        self.parent_resource.set_deletion_policy(resource.RETAIN)
-        for res in self.stack.resources.values():
-            self.assertEqual(resource.RETAIN, res.t['DeletionPolicy'])
-
-    @utils.stack_delete_after
-    def test_get_abandon_data(self):
-        self.parent_resource.create_with_template(self.templ,
-                                                  {"KeyName": "key"})
-        ret = self.parent_resource.get_abandon_data()
+        ret = self.parent_resource.prepare_abandon()
         # check abandoned data contains all the necessary information.
         # (no need to check stack/resource IDs, because they are
         # randomly generated uuids)
-        self.assertEqual(6, len(ret))
+        self.assertEqual(9, len(ret))
         self.assertEqual('CREATE', ret['action'])
         self.assertIn('name', ret)
         self.assertIn('id', ret)
         self.assertIn('resources', ret)
         self.assertEqual(template_format.parse(param_template),
                          ret['template'])
+        self.assertIn('stack_user_project_id', ret)
+        self.assertIn('project_id', ret)
+        self.assertIn('environment', ret)
 
-    @utils.stack_delete_after
     def test_create_with_template_validates(self):
         """
         Creating a stack with a template validates the created stack, so that
@@ -281,7 +360,6 @@ class StackResourceTest(HeatTestCase):
             self.parent_resource.create_with_template,
             template, {'WebServer': 'foo'})
 
-    @utils.stack_delete_after
     def test_update_with_template_validates(self):
         """Updating a stack with a template validates the created stack."""
         create_result = self.parent_resource.create_with_template(
@@ -296,7 +374,6 @@ class StackResourceTest(HeatTestCase):
             self.parent_resource.update_with_template,
             template, {'WebServer': 'foo'})
 
-    @utils.stack_delete_after
     def test_update_with_template_ok(self):
         """
         The update_with_template method updates the nested stack with the
@@ -326,7 +403,6 @@ class StackResourceTest(HeatTestCase):
             self.parent_stack.context, self.stack.id)
         self.assertEqual(self.parent_stack.id, saved_stack.owner_id)
 
-    @utils.stack_delete_after
     def test_update_with_template_timeout_mins(self):
         self.assertIsNone(self.parent_stack.timeout_mins)
         self.m.StubOutWithMock(self.parent_stack, 'timeout_mins')
@@ -349,7 +425,6 @@ class StackResourceTest(HeatTestCase):
         self.assertEqual(200, self.stack.timeout_mins)
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_update_with_template_files(self):
         create_result = self.parent_resource.create_with_template(
             self.simple_template, {})
@@ -367,7 +442,6 @@ class StackResourceTest(HeatTestCase):
 
         self.assertEqual({"foo": "bar"}, self.stack.t.files)
 
-    @utils.stack_delete_after
     def test_update_with_template_state_err(self):
         """
         update_with_template_state_err method should raise error when update
@@ -397,11 +471,10 @@ class StackResourceTest(HeatTestCase):
         ex = self.assertRaises(exception.Error,
                                self.parent_resource.check_update_complete,
                                updater)
-        self.assertEqual('Nested stack UPDATE failed: ', str(ex))
+        self.assertEqual('Nested stack UPDATE failed: ', six.text_type(ex))
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_load_nested_ok(self):
         self.parent_resource.create_with_template(self.templ,
                                                   {"KeyName": "key"})
@@ -418,7 +491,6 @@ class StackResourceTest(HeatTestCase):
         self.parent_resource.nested()
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
     def test_load_nested_non_exist(self):
         self.parent_resource.create_with_template(self.templ,
                                                   {"KeyName": "key"})
@@ -486,7 +558,45 @@ class StackResourceTest(HeatTestCase):
 
         self.m.VerifyAll()
 
-    @utils.stack_delete_after
+    def test_resolve_attribute_string(self):
+        nested = self.m.CreateMockAnything()
+        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
+        stack_resource.StackResource.nested().AndReturn(nested)
+        nested.outputs = {'key': 'value'}
+        nested.output('key').AndReturn('value')
+        self.m.ReplayAll()
+
+        self.assertEqual('value',
+                         self.parent_resource._resolve_attribute("key"))
+
+        self.m.VerifyAll()
+
+    def test_resolve_attribute_dict(self):
+        nested = self.m.CreateMockAnything()
+        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
+        stack_resource.StackResource.nested().AndReturn(nested)
+        nested.outputs = {'key': {'a': 1, 'b': 2}}
+        nested.output('key').AndReturn({'a': 1, 'b': 2})
+        self.m.ReplayAll()
+
+        self.assertEqual({'a': 1, 'b': 2},
+                         self.parent_resource._resolve_attribute("key"))
+
+        self.m.VerifyAll()
+
+    def test_resolve_attribute_list(self):
+        nested = self.m.CreateMockAnything()
+        self.m.StubOutWithMock(stack_resource.StackResource, 'nested')
+        stack_resource.StackResource.nested().AndReturn(nested)
+        nested.outputs = {"key": [1, 2, 3]}
+        nested.output('key').AndReturn([1, 2, 3])
+        self.m.ReplayAll()
+
+        self.assertEqual([1, 2, 3],
+                         self.parent_resource._resolve_attribute("key"))
+
+        self.m.VerifyAll()
+
     def test_create_complete_state_err(self):
         """
         check_create_complete should raise error when create task is
@@ -497,14 +607,12 @@ class StackResourceTest(HeatTestCase):
 
         ctx = self.parent_resource.context
         phy_id = "cb2f2b28-a663-4683-802c-4b40c916e1ff"
-        templ = parser.Template(self.templ)
+        templ = templatem.Template(self.templ)
         env = environment.Environment({"KeyName": "test"})
         self.stack = parser.Stack(ctx, phy_id, templ, env, timeout_mins=None,
                                   disable_rollback=True,
-                                  parent_resource=self.parent_resource)
-
-        self.m.StubOutWithMock(parser, 'Template')
-        parser.Template(self.templ, files={}).AndReturn(templ)
+                                  parent_resource=self.parent_resource,
+                                  stack_user_project_id='aprojectid')
 
         self.m.StubOutWithMock(environment, 'Environment')
         environment.Environment().AndReturn(env)
@@ -514,7 +622,9 @@ class StackResourceTest(HeatTestCase):
                      disable_rollback=True,
                      parent_resource=self.parent_resource,
                      owner_id=self.parent_stack.id,
-                     adopt_stack_data=None).AndReturn(self.stack)
+                     user_creds_id=self.parent_stack.user_creds_id,
+                     adopt_stack_data=None,
+                     stack_user_project_id='aprojectid').AndReturn(self.stack)
 
         st_set = self.stack.state_set
         self.m.StubOutWithMock(self.stack, 'state_set')
@@ -535,7 +645,6 @@ class StackResourceTest(HeatTestCase):
         # Restore state_set to let clean up proceed
         self.stack.state_set = st_set
 
-    @utils.stack_delete_after
     def test_suspend_complete_state_err(self):
         """
         check_suspend_complete should raise error when suspend task is
@@ -565,7 +674,6 @@ class StackResourceTest(HeatTestCase):
         # Restore state_set to let clean up proceed
         self.stack.state_set = st_set
 
-    @utils.stack_delete_after
     def test_resume_complete_state_err(self):
         """
         check_resume_complete should raise error when resume task is
@@ -596,3 +704,15 @@ class StackResourceTest(HeatTestCase):
         self.m.VerifyAll()
         # Restore state_set to let clean up proceed
         self.stack.state_set = st_set
+
+    def test_check_nested_resources(self):
+        def _mock_check(res):
+            res.handle_check = mock.Mock()
+
+        self.parent_resource.create_with_template(self.templ, {"KeyName": "k"})
+        nested = self.parent_resource.nested()
+        [_mock_check(res) for res in nested.resources.values()]
+
+        scheduler.TaskRunner(self.parent_resource.check)()
+        [self.assertTrue(res.handle_check.called)
+         for res in nested.resources.values()]
